use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Pithub.pm',
    'lib/Pithub/Base.pm',
    'lib/Pithub/Events.pm',
    'lib/Pithub/Gists.pm',
    'lib/Pithub/Gists/Comments.pm',
    'lib/Pithub/GitData.pm',
    'lib/Pithub/GitData/Blobs.pm',
    'lib/Pithub/GitData/Commits.pm',
    'lib/Pithub/GitData/References.pm',
    'lib/Pithub/GitData/Tags.pm',
    'lib/Pithub/GitData/Trees.pm',
    'lib/Pithub/Issues.pm',
    'lib/Pithub/Issues/Assignees.pm',
    'lib/Pithub/Issues/Comments.pm',
    'lib/Pithub/Issues/Events.pm',
    'lib/Pithub/Issues/Labels.pm',
    'lib/Pithub/Issues/Milestones.pm',
    'lib/Pithub/Markdown.pm',
    'lib/Pithub/Orgs.pm',
    'lib/Pithub/Orgs/Members.pm',
    'lib/Pithub/Orgs/Teams.pm',
    'lib/Pithub/PullRequests.pm',
    'lib/Pithub/PullRequests/Comments.pm',
    'lib/Pithub/PullRequests/Reviewers.pm',
    'lib/Pithub/Repos.pm',
    'lib/Pithub/Repos/Collaborators.pm',
    'lib/Pithub/Repos/Commits.pm',
    'lib/Pithub/Repos/Contents.pm',
    'lib/Pithub/Repos/Downloads.pm',
    'lib/Pithub/Repos/Forks.pm',
    'lib/Pithub/Repos/Hooks.pm',
    'lib/Pithub/Repos/Keys.pm',
    'lib/Pithub/Repos/Releases.pm',
    'lib/Pithub/Repos/Releases/Assets.pm',
    'lib/Pithub/Repos/Starring.pm',
    'lib/Pithub/Repos/Stats.pm',
    'lib/Pithub/Repos/Statuses.pm',
    'lib/Pithub/Repos/Watching.pm',
    'lib/Pithub/Result.pm',
    'lib/Pithub/Result/SharedCache.pm',
    'lib/Pithub/ResultSet.pm',
    'lib/Pithub/Search.pm',
    'lib/Pithub/SearchV3.pm',
    'lib/Pithub/Users.pm',
    'lib/Pithub/Users/Emails.pm',
    'lib/Pithub/Users/Followers.pm',
    'lib/Pithub/Users/Keys.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/basic.t',
    't/encoding.t',
    't/events.t',
    't/gists.t',
    't/git_data.t',
    't/issues.t',
    't/lib/Pithub/Test.pm',
    't/lib/Pithub/Test/Factory.pm',
    't/lib/Pithub/Test/UA.pm',
    't/lib/Pithub/Test/http_response/api.github.com/error/notfound.GET',
    't/lib/Pithub/Test/http_response/api.github.com/orgs/CPAN-API/repos.GET',
    't/lib/Pithub/Test/http_response/api.github.com/repos/foo/bar/downloads.POST',
    't/lib/Pithub/Test/http_response/api.github.com/repos/foo/bar/issues.GET',
    't/lib/Pithub/Test/http_response/api.github.com/user/following/rafl.GET',
    't/lib/Pithub/Test/http_response/api.github.com/users/miyagawa/followers.GET',
    't/lib/Pithub/Test/http_response/api.github.com/users/miyagawa/followers.GET.page-26',
    't/lib/Pithub/Test/http_response/api.github.com/users/miyagawa/followers.GET.page-3',
    't/lib/Pithub/Test/http_response/api.github.com/users/miyagawa/followers.GET.per_page-1',
    't/lib/Pithub/Test/http_response/api.github.com/users/plu.GET',
    't/lib/Pithub/Test/http_response/api.github.com/users/plu/followers.GET.page-2.per_page-15',
    't/lib/Pithub/Test/http_response/api.github.com/users/plu/followers.GET.page-3.per_page-15',
    't/lib/Pithub/Test/http_response/api.github.com/users/plu/followers.GET.page-4.per_page-15',
    't/lib/Pithub/Test/http_response/api.github.com/users/plu/followers.GET.per_page-15',
    't/lib/Pithub/Test/http_response/api.github.com/users/plu/repos.GET',
    't/lib/Pithub/Test/http_response/api.github.com/users/rwstauner.GET',
    't/live/basic.t',
    't/live/cache-lru.t',
    't/live/cache.t',
    't/live/events.t',
    't/live/gists.t',
    't/live/git_data.t',
    't/live/issues.t',
    't/live/orgs.t',
    't/live/pull_requests.t',
    't/live/repos.t',
    't/live/users.t',
    't/multi.t',
    't/orgs.t',
    't/pull_requests.t',
    't/repos.t',
    't/search.t',
    't/test.t',
    't/users.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
